## translationsPage.py - show selinux translations
## Copyright (C) 2006 Red Hat, Inc.

## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

## Author: Dan Walsh
import string
import gtk
import gtk.glade
import os
import gobject
import sys
import seobject
from semanagePage import *;

##
## I18N
## 
PROGNAME="policycoreutils"
import gettext
gettext.bindtextdomain(PROGNAME, "/usr/share/locale")
gettext.textdomain(PROGNAME)
try:
    gettext.install(PROGNAME, localedir="/usr/share/locale", unicode=1)
except IOError:
    import __builtin__
    __builtin__.__dict__['_'] = unicode

class translationsPage(semanagePage):
    def __init__(self, xml):
        semanagePage.__init__(self, xml, "translations", _("Translation"))
        self.firstTime = False

        self.translation_filter = xml.get_widget("translationsFilterEntry")
        self.translation_filter.connect("focus_out_event", self.filter_changed)
        self.translation_filter.connect("activate", self.filter_changed)

        self.store = gtk.ListStore(gobject.TYPE_STRING, gobject.TYPE_STRING)
        self.view.set_model(self.store)
        self.store.set_sort_column_id(0, gtk.SORT_ASCENDING)        
        col = gtk.TreeViewColumn(_("Sensitvity Level"), gtk.CellRendererText(), text = 0)
        col.set_sort_column_id(0)
	col.set_sizing(gtk.TREE_VIEW_COLUMN_FIXED)
        col.set_resizable(True)
        col.set_fixed_width(250)
        self.view.append_column(col)
        col = gtk.TreeViewColumn(_("Translation"), gtk.CellRendererText(), text = 1)
        col.set_sort_column_id(1)
        col.set_resizable(True)
        self.view.append_column(col)

        self.load()
        self.translationsLevelEntry = xml.get_widget("translationsLevelEntry")
        self.translationsEntry = xml.get_widget("translationsEntry")
        
    def load(self, filter = ""):
        self.filter = filter
        self.translation = seobject.setransRecords()
        dict = self.translation.get_all()
        keys = dict.keys()
        keys.sort()
        self.store.clear()
        for k in keys:
            if not (self.match(k, filter) or self.match(dict[k], filter)):
                continue
            iter = self.store.append()
            self.store.set_value(iter, 0, k)
            self.store.set_value(iter, 1, dict[k])
        self.view.get_selection().select_path ((0,))

    def dialogInit(self):
        store, iter = self.view.get_selection().get_selected()
        self.translationsLevelEntry.set_text(store.get_value(iter, 0))
        self.translationsLevelEntry.set_sensitive(False)
        self.translationsEntry.set_text(store.get_value(iter, 1))
        
    def dialogClear(self):
        self.translationsLevelEntry.set_text("")
        self.translationsLevelEntry.set_sensitive(True)
        self.translationsEntry.set_text("")
        
    def delete(self):
        store, iter = self.view.get_selection().get_selected()
        try:
            level = store.get_value(iter, 0)
            self.translation.delete(level)
            store.remove(iter)
            self.view.get_selection().select_path ((0,))
        except ValueError, e:
            self.error(e.args[0])

    def add(self):
        level = self.translationsLevelEntry.get_text().strip()
        translation = self.translationsEntry.get_text().strip()
        self.translation.add(level, translation)
        iter = self.store.append()
        self.store.set_value(iter, 0, level)
        self.store.set_value(iter, 1, translation)
        
    def modify(self):
        level = self.translationsLevelEntry.get_text().strip()
        translation = self.translationsEntry.get_text().strip()
        self.translation.modify(level, translation)
        store, iter = self.view.get_selection().get_selected()
        self.store.set_value(iter, 0, level)
        self.store.set_value(iter, 1, translation)
